@@ -1,7 +1,7 @@
 #
 #       images.R
 #
-#      $Revision: 1.169 $     $Date: 2021/06/23 02:19:29 $
+#      $Revision: 1.171 $     $Date: 2022/03/24 03:22:07 $
 #
 #      The class "im" of raster images
 #
@@ -788,7 +788,9 @@ safelookup <- function(Z, x, factor=2, warn=TRUE) {
   Zvals[isna] <- Nvals
   if(all(fixed))
     return(Zvals)
-  isna[isna] <- !fixed
+  notfixed <- !fixed
+  isna[isna] <- notfixed
+  Xbad <- Xbad[notfixed]
   #' Third pass - project to nearest pixel at any distance
   W <- as.mask(Z)
   eW <- exactPdt(W)
@@ -1353,3 +1355,18 @@ fillNA <- function(x, value=0) {
   x$v <- v
   return(x)
 }
+
+
+## probability integral transformation
+## aka histogram equalisation
+## aka transformation to uniformity
+
+transformquantiles <- function(X, uniform=FALSE, reverse=FALSE, ...) {
+  if(!uniform && !reverse) return(X)
+  o <- order(X[])
+  V <- X
+  n <- length(o)
+  if(uniform) V[][o] <- (seq_len(n) - 0.5)/n
+  if(reverse) V[][o] <- V[][rev(o)]
+  return(V)
+}
